/***************************************************************

  BayCom(R)           Packet-Radio fuer IBM PC

  BayCom-Terminal
  BayCom-Mailbox

  Routinen fuer Mailboxsystem


  -----------------------------
  Terminal-Emulation fuer Linux
  -----------------------------


  Copyright (C)       Florian Radlherr
                      Taubenbergstr. 32
                      83627 Warngau

  Alle Rechte vorbehalten / All Rights Reserved

 ***************************************************************/

#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <unistd.h>
#include <curses.h>
#include "term_l.h"

WINDOW *wsend;
WINDOW *wrec;
WINDOW *wstat;

#define TXWINLEN 4
#define KEY_CR 13
#define PUTBUF 10000

/*---------------------------------------------------------------------------*/

static char putbuf[PUTBUF];
int putbufindex=0;

#define TX_SCROLL 100
#define LINELEN 80

char tx_scrollbuf[TX_SCROLL*LINELEN*2];
int tx_scrolldepth=0;
int tx_scrollpos=0;
int tx_pos=0;

/*---------------------------------------------------------------------------*/

void putflush_cur(void)
{ if(putbufindex)
  {
    putbuf[putbufindex]=0;
    waddstr(wrec,putbuf);
    putbufindex=0;
    wrefresh(wrec);
    wrefresh(wsend);
  }
} 

/*---------------------------------------------------------------------------*/

void putv_cur(int ch)
{
  if(ch==13)
    return;

  switch((unsigned char)ch)
  { 
    case 0x84: putv_cur('a'); ch='e'; break;
    case 0x94: putv_cur('o'); ch='e'; break;
    case 0x81: putv_cur('u'); ch='e'; break;
    case 0x8e: putv_cur('A'); ch='e'; break;
    case 0x99: putv_cur('O'); ch='e'; break;
    case 0x9a: putv_cur('U'); ch='e'; break;
    case 0xe1: putv_cur('s'); ch='s'; break;
    /*
    case 0x84: ch=''; break;
    case 0x94: ch=''; break;
    case 0x81: ch=''; break;
    case 0x8e: ch=''; break;
    case 0x99: ch=''; break;
    case 0x9a: ch=''; break;
    case 0xe1: ch=''; break;
    */
  }
  if(ch>128 || (ch<32 && ch!=10)) 
    ch='.';
    
  putbuf[putbufindex]=ch;
  putbufindex++;
  if(putbufindex==PUTBUF)
    putflush_cur();
}

/*---------------------------------------------------------------------------*/

void putf(char *format, ...)
{ va_list argpoint;
  char cbuf[260];
  char *s=cbuf;

  va_start(argpoint,format);
  vsprintf(cbuf,format,argpoint);
  va_end(argpoint);
  if(strlen(s)>257)
    endmessage("putf","long %s",s);
  
  while(*s)
    putv_cur(*(s++));
}

void statst(int pos,char *str)
{  
  wattron(wstat,A_REVERSE);
  if(str==NULL)
  { 
    wmove(wstat,0,0);
    waddstr(wstat,"                                                                                ");
  }
  else
  { 
    wmove(wstat,0,pos);
    waddstr(wstat,str);

  }
    
  wrefresh(wstat);
  wrefresh(wsend);
}


void initvideo(void)
{
  initscr();
  nonl();
  cbreak();
  noecho();

  wstat=newwin(1,0,TXWINLEN,0);
  wrec=newwin(0,0,TXWINLEN+1,0);
  wsend=newwin(TXWINLEN,0,0,0);

  idlok(wrec,1);
  idlok(wsend,1);
  scrollok(wrec,1);
  scrollok(wsend,1);
  nodelay(wrec,1);
  nodelay(wsend,1);
  keypad(wsend,1);
  meta(wsend,1);
  atexit(endvideo);

  statst(0,NULL);
}

void endvideo(void)
{
  wclear(wsend);
  wclear(wrec);
  wclear(wstat);
  endwin();
}

#define LAST_CMDS 10
static char *lastcmd[LAST_CMDS];
static int next_cmd,walk_cmd,nr_cmds;

static void LcmdRegister(char *line)
   {
   int cnt,have_it;

   have_it=0;
   if (line==NULL || strlen(line)==0) have_it=1;
   if (have_it==0)
      {
      cnt=(next_cmd>0) ? next_cmd-1 : LAST_CMDS-1;
      if (lastcmd[cnt] && 0==strcmp(lastcmd[cnt],line)) { have_it=1; }
      }

   if (have_it==0)
      {
      free(lastcmd[next_cmd]);
      lastcmd[next_cmd]=(char*)malloc(strlen(line)+1);
      strcpy(lastcmd[next_cmd],line);
      next_cmd++;
      if (next_cmd>=LAST_CMDS) next_cmd=0;
      nr_cmds++;
      if (nr_cmds>=LAST_CMDS) nr_cmds=LAST_CMDS;
      }

   walk_cmd=next_cmd;
   }

static char *LcmdGet(int nr)
   {
   char *retc;

   retc=NULL;
   if (nr>=0 && nr<LAST_CMDS) { retc=lastcmd[nr]; }

   return(retc);
   }

static int LcmdReplace(int nr,char *line,int txpos,int *korr)
   {
   char *lcmd;
   int retc,l1,l2;

   retc=txpos;
   lcmd=LcmdGet(nr);
   if (lcmd) 
      { 
      if (korr)
         {
         l1=strlen(line);
         l2=strlen(lcmd);
         if (l2<l1) *korr+=(l1-l2);
         }
      strcpy(line,lcmd); retc=strlen(line); 
      }
   return(retc);
   }

int scankey(char *ret_line)
{ int c;
  int y,x;
  //char *line=tx_scrollbuffer+(LINELEN*tx_scrollpos);
  char *line=ret_line;
  int korr;

  korr=1;
  getyx(wsend,y,x);

  c=wgetch(wsend);
  switch(c)
  { 
    case ERR: 
    	return 0;
    case KEY_CR: 
    	wmove(wsend,y,0);
	wprintw(wsend,"%s\n",line);
    	wprintw(wrec,"%s\n",line);
    	wrefresh(wsend);
	tx_pos=0;
        LcmdRegister(line);
    	return 1;
    case KEY_BACKSPACE:
        if(tx_pos)
	{ 
	  line[LINELEN-2]=' ';
	  tx_pos--;
	  memmove(line+tx_pos,line+tx_pos+1,LINELEN-(tx_pos+1));
	}
	break;

    case KEY_UP:
       walk_cmd--;
       if (walk_cmd<0) walk_cmd=nr_cmds-1;
       if (walk_cmd<0) walk_cmd=0;
       tx_pos=LcmdReplace(walk_cmd,line,tx_pos,&korr);

    	if(tx_scrolldepth>tx_scrollpos)
	{ tx_scrollpos++;
	}
	break;
    case KEY_DOWN:
       walk_cmd++;
       if (walk_cmd>=nr_cmds) walk_cmd=0;
       tx_pos=LcmdReplace(walk_cmd,line,tx_pos,&korr);

    	if(tx_scrollpos)
	{ tx_scrollpos--;
	}
	break;
    case KEY_LEFT:
    	if(tx_pos)
	  tx_pos--;
	break; 
	
    case KEY_RIGHT:
        if(tx_pos<80)
	  tx_pos++;
        break;
	
    default:
      if (tx_pos < 79)
      {  memmove(line+tx_pos+1,line+tx_pos,LINELEN-(tx_pos+1));
        line[tx_pos]=c;
        tx_pos++;
      }
  }  
  //line=tx_scrollbuffer+(LINELEN*tx_scrollpos);
  wmove(wsend,y,0);
  waddstr(wsend,line);
  while(korr>0)
     {
     waddstr(wsend," ");
     korr--;
     }
  wmove(wsend,y,tx_pos);
  wrefresh(wsend);
  return 0;
}

